#ifndef _BOARD_H_
#define _BOARD_H_


#include "pattern.h"


#define TRUE  1
#define FALSE 0

enum STONE {
	S_EMPTY,	/* _ */
	S_BLACK,	/* @ */
	S_WHITE,	/* @ */
	S_OB,		/* ՊO */
	S_MAX,
};

#define OTHER_COLOR( color ) ((color)^0x3)	/* F] */

#define PASS   0
#define RESIGN -1

#define KOMI 6.5	/* R~ */

#define PURE_BOARD_SIZE	9	/* Ղ̖{̑傫 */
#define OB_SIZE 2			/* ՊȎ傫 */
#define BOARD_SIZE	(PURE_BOARD_SIZE + (OB_SIZE + OB_SIZE)) /* Ղ̑傫 */

#define BOARD_START OB_SIZE
#define BOARD_END	(OB_SIZE+PURE_BOARD_SIZE-1)

#define PURE_BOARD_MAX	(PURE_BOARD_SIZE * PURE_BOARD_SIZE)
#define BOARD_MAX		(BOARD_SIZE * BOARD_SIZE)

#define MAX_STRINGS (PURE_BOARD_SIZE * PURE_BOARD_SIZE * 4/5)	/* A̍ő吔 */

#define MAX_LIBERTIES 3
#define MAX_NEIGHBORS MAX_STRINGS

#define MAX_RECORDS 1000
#define MAX_MOVES MAX_RECORDS-1	/* ő萔 */

#define POS(x, y)     ((x) + (y) * BOARD_SIZE)
#define X(pos)        ((pos) % BOARD_SIZE)
#define Y(pos)        ((pos) / BOARD_SIZE)
#define NORTH(pos)    ((pos) - BOARD_SIZE)
#define EAST(pos)     ((pos) + 1)
#define SOUTH(pos)    ((pos) + BOARD_SIZE)
#define WEST(pos)     ((pos) - 1)

/* ߖT̍W */
#define NE  (-BOARD_SIZE+1)
#define NW  (-BOARD_SIZE-1)
#define SE  (BOARD_SIZE+1)
#define SW  (BOARD_SIZE-1)

#define NN  (-BOARD_SIZE-BOARD_SIZE)
#define SS  (BOARD_SIZE+BOARD_SIZE)
#define EE  2
#define WW  (-2)

/* ̋ */
#define DX(pos1,pos2)	(abs(board_x[pos1]-board_x[pos2]))
#define DY(pos1,pos2)	(abs(board_y[pos1]-board_y[pos2]))
#define DIS(pos1,pos2)	(move_dis[DX(pos1,pos2)][DY(pos1,pos2)])


/*------------------------------------------------------------------*/
/* \                                                           */
/*------------------------------------------------------------------*/

/*  */
struct move {
	int color;
	int pos;
};

/*  */
struct stone {
	int color;
	int string_number;
	int next_stone;
	int mark;
};

/* A */
struct string {
	int color;
	int size;
	int origin;
	int liberties;
	int neighbors;
	int mark;
};

/* _ */
struct string_liberties {
	int list[4];
};

/* אڂG̘A */
struct string_neighbors {
	int list[MAX_NEIGHBORS];
};

/* ΋ǂ̑S */
struct game {
	int prisoner[S_MAX];

	int moves;				
	struct move record[MAX_RECORDS];

	int ko_pos;
	int ko_move;

	struct stone board[BOARD_MAX];

	struct string string[MAX_STRINGS];
	struct string_liberties string_liberties[MAX_STRINGS];
	struct string_neighbors string_neighbors[MAX_STRINGS];
	int next_string;
	int liberty_mark;
	int string_mark;

	int pat3[BOARD_MAX];
	int dia[BOARD_MAX];

	int candidates[BOARD_MAX]; 

	int rate[BOARD_MAX];
	int sum_rate_column[BOARD_SIZE];
	int sum_rate;
};


/*-------------------------------------------------------------------*/
/* O[oϐ                                                    */
/*-------------------------------------------------------------------*/

/* R~ */
extern double komi[3];

/* Տ̍W */
extern int onboard_pos[PURE_BOARD_MAX];

/* XW */
extern int board_x[BOARD_MAX];

/* YW */
extern int board_y[BOARD_MAX];

/* 4ߖT̋_ */
extern int nb4_empty[PATTERN3_MAX];

/*  */
extern unsigned char eye[PATTERN3_MAX];

/*   */
extern unsigned char border_dis_x[BOARD_MAX];
extern unsigned char border_dis_y[BOARD_MAX];
extern unsigned char move_dis[PURE_BOARD_SIZE][PURE_BOARD_SIZE];;


/*-------------------------------------------------------------------*/
/* ֐                                                              */
/*-------------------------------------------------------------------*/

/*  (m + ݒ) */
struct game *InitializeGame( void );

/* m */
struct game *SetupGame( void );

/* ݒ */
void ClearGame( struct game *game );

/* Rs[ */
void CopyGame( struct game *g2, struct game *g1 );

/*  */
void DoneGame( struct game *game );

/* 萔̐ݒ */
void BoardConst( void );

/* @̔*/
int CheckLegal( struct game *game, int color, int pos );

/* @肩łȂ */
int CheckLegalAndEye( struct game *game, int color, int pos );

/* Ղɐ΂u*/
void SetStone( struct game *game, int color, int pos );

/* s̔ */
int BoardScore( struct game *game );

/* Ղ\ */
void DisplayBoard( struct game *game  );


#endif  /* _BOARD_H_ */